package stockMarketApplet.GUI;

import java.awt.*;
import java.util.*;
import java.applet.*;
import javax.swing.JFrame;

import stockMarketApplet.GUI.*;
import stockMarketApplet.util.*;
import stockMarketApplet.market.*;
import stockMarketApplet.market.io.*;

public class OptFundamentalFrame extends Frame {

  private GridBagLayout gridbag;
  private Panel lpanel, rpanel, cpanel, buttonpanel;

  private Scroller cescroller;
  private Scroller sescroller;

  private TexFld initialValueFld;
  private TexFld meanFld;
  private TexFld standardDevFld;

  private TexFld TminFld;
  private TexFld TmaxFld;
  private TexFld TauMinFld;
  private TexFld TauMaxFld;
  private TexFld aFld;

  private FundamentalParametersRecord fundparamRec;

  private Button okbutton;
  private Button cancelbutton;
  private Market market;
  private OptFrame frame;

  // Create an instance of OptFundamental Frame, using the provided String as title, the Market object
  // as reference to the market whose value are to be changed and a reference to the parent Applet
  public OptFundamentalFrame(String title, Market market, OptFrame frame)
  {
    super(title);
    this.setBackground(Color.lightGray);
    this.market = market;
    this.frame = frame;
    gridbag= new GridBagLayout();
    setLayout(gridbag);
    Util util = new Util();
    lpanel= new Panel();
    lpanel.setLayout(gridbag);
    rpanel= new Panel();
    rpanel.setLayout(gridbag);
    cpanel= new Panel();
    cpanel.setLayout(gridbag);
    buttonpanel= new Panel();
    buttonpanel.setLayout(gridbag);
    GridBagConstraints c = new GridBagConstraints();

    this.readParameters();

    cescroller = new Scroller("Initial cash endowment",1,100010, (int) fundparamRec.getCash());
    util.constraints(lpanel, cescroller, c, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    sescroller = new Scroller("Initial endowment of shares",1,1010, (int) fundparamRec.getNumberShares());
    util.constraints(lpanel, sescroller, c, 0, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);

    initialValueFld = new TexFld("Initial Fundamental Value",""+fundparamRec.getInitialValue(),5);
    util.constraints(cpanel, initialValueFld, c, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    meanFld = new TexFld("Mean",""+fundparamRec.getMean(),5);
    util.constraints(cpanel, meanFld, c, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    standardDevFld = new TexFld("Standard Deviation",""+fundparamRec.getStandardDeviation(),5);
    util.constraints(cpanel, standardDevFld, c, 0, 2, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    aFld = new TexFld ("Position's constant a",""+fundparamRec.getA(),5);
    util.constraints(cpanel, aFld, c, 0, 3, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);


    TminFld = new TexFld("Tmin",""+fundparamRec.getTmin(),5);
    util.constraints(rpanel, TminFld, c, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    TmaxFld = new TexFld("Tmax",""+fundparamRec.getTmax(),5);
    util.constraints(rpanel, TmaxFld, c, 0, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    TauMinFld = new TexFld("TauMin",""+fundparamRec.getTauMin(),5);
    util.constraints(rpanel, TauMinFld, c, 0, 2, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    TauMaxFld = new TexFld("TauMax",""+fundparamRec.getTauMax(),5);
    util.constraints(rpanel, TauMaxFld, c, 0, 3, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);


    util.constrain(buttonpanel, okbutton= new Button("Ok"), 1, 0, 1, 1, 5, 10, 0, 0);
    util.constrain(buttonpanel, cancelbutton= new Button("Cancel"), 2, 0, 1, 1, 5, 10, 0, 0);

    util.constrain(this, new Label("Fundamental Traders Parameters"), 0, 0, 5, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 0, 200, 0, 200);
    util.constrain(this, lpanel, 0, 1, 1, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 0, 0, 0, 0);
    util.constrain(this, cpanel, 1, 1, 1, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 0, 0, 0, 0);
    util.constrain(this, rpanel, 2, 1, 1, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 0, 0, 0, 0);
    util.constrain(this, buttonpanel, 0, 2, 3, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 1.0, 0, 0, 0, 0);
    pack();
  }

  // Handle input events
  public boolean action(Event e, Object arg)
  {
    if(e.target == okbutton)
    {
      saveParameters();
      this.dispose();
      try{
        this.finalize();
      }catch (Throwable ee )
      {
        System.out.println("Throw exeption ");
      }
      return true;
    } else if(e.target == cancelbutton)
    {
      this.dispose();
      try{
        this.finalize();
      }catch (Throwable ee )
      {
        System.out.println("Throw exeption ");
      }
      return true;
    } else
    return true;
  }

  private void readParameters(){
    fundparamRec = new FundamentalParametersRecord();
    InputFundamentalParameters inputFundParamRec = new InputFundamentalParameters();
    fundparamRec = inputFundParamRec.readParameters();
  }

  private void saveParameters(){
    fundparamRec.setCash(new Double(cescroller.getValue()).doubleValue());
    fundparamRec.setNumberShares(new Double(sescroller.getValue()).doubleValue());
    fundparamRec.setInitialValue(new Double(initialValueFld.getValue()).doubleValue());
    fundparamRec.setMean(new Double(meanFld.getValue()).doubleValue());
    fundparamRec.setStandardDeviation(new Double(standardDevFld.getValue()).doubleValue());
    fundparamRec.setTmin(new Double(TminFld.getValue()).doubleValue());
    fundparamRec.setTmax(new Double(TmaxFld.getValue()).doubleValue());
    fundparamRec.setTauMin(new Double(TauMinFld.getValue()).doubleValue());
    fundparamRec.setTauMax(new Double(TauMaxFld.getValue()).doubleValue());
    fundparamRec.setA(new Double(aFld.getValue()).doubleValue());
    OutputFundamentalParameters outFundParam = new OutputFundamentalParameters();
    outFundParam.saveParameters(fundparamRec);
  };

}