package stockMarketApplet.GUI;

import java.awt.*;
import java.util.*;
import java.applet.*;
import javax.swing.JFrame;

import stockMarketApplet.GUI.*;
import stockMarketApplet.util.*;
import stockMarketApplet.market.*;
import stockMarketApplet.market.io.*;

// This class opens a window with several scrollbars and an optional List.
// When the OK button is pressed, the changed values are directly set in
// the assigned Market
public class OptNoisyFrame extends Frame {

  private GridBagLayout gridbag;
  private Panel lpanel, rpanel, buttonpanel;

  private Scroller cescroller;
  private Scroller sescroller;

  private TexFld probBuyFld;
  private TexFld probSellFld;
  private TexFld probHoldFld;

  private Button okbutton;
  private Button cancelbutton;
  private Market market;
  private OptFrame frame;

  private NoisyTraderParametersRecord noisyParamRec;

  // Create an instance of OptNoisy Frame, using the provided String as title, the Market object
  // as reference to the market whose value are to be changed and a reference to the parent Applet
  public OptNoisyFrame(String title, Market market, OptFrame frame)
  {
    super(title);
    this.setBackground(Color.lightGray);
    this.market = market;
    this.frame = frame;
    gridbag= new GridBagLayout();
    setLayout(gridbag);
    Util util = new Util();
    lpanel= new Panel();
    lpanel.setLayout(gridbag);
    rpanel= new Panel();
    rpanel.setLayout(gridbag);
    buttonpanel= new Panel();
    buttonpanel.setLayout(gridbag);
    GridBagConstraints c = new GridBagConstraints();

    this.readParameters();

    cescroller = new Scroller("Initial cash endowment",1,100010, (int) noisyParamRec.getCash());
    util.constraints(lpanel, cescroller, c, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    sescroller = new Scroller("Initial endowment of shares",1,1010, (int) noisyParamRec.getNumberShares());
    util.constraints(lpanel, sescroller, c, 0, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);

    probBuyFld = new TexFld("Probability to Buy",""+noisyParamRec.getProbabilityBuy(),5);
    util.constraints(rpanel, probBuyFld, c, 0, 2, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    probSellFld = new TexFld("Probability to Sell",""+noisyParamRec.getProbabilitySell(),5);
    util.constraints(rpanel, probSellFld, c, 0, 3, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);
    probHoldFld = new TexFld("Probability to Hold",""+noisyParamRec.getProbabilityHold(),5);
    util.constraints(rpanel, probHoldFld, c, 0, 4, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 1);

    util.constrain(buttonpanel, okbutton= new Button("Ok"), 1, 0, 1, 1, 5, 10, 0, 0);
    util.constrain(buttonpanel, cancelbutton= new Button("Cancel"), 2, 0, 1, 1, 5, 10, 0, 0);

    util.constrain(this, new Label("Noisy Traders Parameters"), 0, 0, 5, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 0, 200, 0, 200);
    util.constrain(this, lpanel, 0, 1, 1, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 0, 0, 0, 0);
    util.constrain(this, rpanel, 1, 1, 1, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 0.0, 0, 0, 0, 0);
    util.constrain(this, buttonpanel, 0, 2, 3, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1.0, 1.0, 0, 0, 0, 0);
    pack();
  }

  // Handle input events
  public boolean action(Event e, Object arg)
  {
    if(e.target == okbutton)
    {
      saveParameters();
      this.dispose();
      try{
        this.finalize();
      }catch (Throwable ee )
      {
        System.out.println("Throw exeption ");
      }
      return true;
    } else if(e.target == cancelbutton)
    {
      this.dispose();
      try{
        this.finalize();
      }catch (Throwable ee )
      {
        System.out.println("Throw exeption ");
      }
      return true;
    } else
    return true;
  }

  private void readParameters(){
    noisyParamRec = new NoisyTraderParametersRecord();
    InputNoisyTradersParameters inputNoisyParam = new InputNoisyTradersParameters();
    noisyParamRec = inputNoisyParam.readParameters();
  }

  private void saveParameters(){
    noisyParamRec.setCash(new Double(cescroller.getValue()).doubleValue());
    noisyParamRec.setNumberShares(new Double(sescroller.getValue()).doubleValue());
    noisyParamRec.setProbabilityBuy(new Double(probBuyFld.getValue()).doubleValue());
    noisyParamRec.setProbabilitySell(new Double(probSellFld.getValue()).doubleValue());
    noisyParamRec.setProbabilityHold(new Double(probHoldFld.getValue()).doubleValue());
    OutputNoisyTraderParameters outNoisyTraderParams = new OutputNoisyTraderParameters();
    outNoisyTraderParams.saveParameters(noisyParamRec);
  };

}