package stockMarketApplet.GUI;

import java.util.*;
import java.text.*;
import java.applet.*;
import java.awt.*;

// PlotCanvas displays three or one graphs, using data that can be set step by step.
// Depending on the mode set in the constructor method, this class plots three or one
// functions in red, green black or just black. These may represent worst, best or
// average values of a variable over the complete generation.
public class PlotCanvas extends Canvas {
  public final static byte THREE = 3;
  public final static byte ONE = 1;
  public final static byte TWO = 2;

  private int mode = 0;
  private String title;

  private Rectangle inb;
  private Font smallfont;
  private FontMetrics metric;
  private int fheight;

  private double minFit, maxFit;
  private int numGens;
  private int age= 0;
  private double[] bests;
  private double[] worsts;
  private double[] averages;

  private boolean limitsset= false;

  // Create a PlotCanvas with mode set to mod and title tit
  public PlotCanvas(int mod, String tit)
  {
    mode= mod;
    title= tit;

    smallfont= new Font("Courier", Font.PLAIN, 12);
    setFont(smallfont);
    metric= this.getFontMetrics(smallfont);
    fheight= metric.getHeight();

    setBackground(Color.white);
    setForeground(Color.black);
  }

  // Set limits for maximum/minium values. Not mandatory, since
  // these values may be determined dynamically, but nice for a
  // maximum number of hits.
  public void setLimits(double minf, double maxf, int gens)
  {
    minFit= minf;
    maxFit= maxf;
    numGens= gens;
    if(mode == THREE)
    {
      bests= new double[numGens +1];
      worsts= new double[numGens +1];
    }
    averages= new double[numGens +1];
    limitsset= true;
  }

  // Set Maximum number of generations
  public void setGenerations(int gens)
  {
    numGens= gens;
    if(mode == THREE)
    {
      bests= new double[numGens +1];
      worsts= new double[numGens +1];
    }
    averages= new double[numGens +1];
  }

  // Add three double values to the three graphs
  public void addValues(int gen, double best, double worst, double average)
  {
    try
    {
      bests[age]= best;
      worsts[age]= worst;
      averages[age]= average;
      age++;
    }
    catch (ArrayIndexOutOfBoundsException e) { }
    catch (NullPointerException n) { }
    repaint();
  }

  // Add one double value to the single graph
  public void addValues(int gen, double average)
  {
    try
    {
      averages[age]= average;
      age++;
    }
    catch (ArrayIndexOutOfBoundsException e) { }
    catch (NullPointerException n) { }
    repaint();
  }

  // Add three int values to the three graphs
  public void addValues(int gen, int best, int worst, int average)
  {
    try
    {
      bests[age]= (double) best;
      worsts[age]= (double) worst;
      averages[age]= (double) average;
      age++;
    }
    catch (ArrayIndexOutOfBoundsException e) { }
    catch (NullPointerException n) { }
    repaint();
  }

  // Add one int value to the single graph
  public void addValues(int gen, int average)
  {
    try
    {
      averages[age]= (double) average;
      age++;
    }
    catch (ArrayIndexOutOfBoundsException e) { }
    catch (NullPointerException n) { }
    repaint();
  }

  // Reset the display to nothing.
  public void clear()
  {
    age= 0;
    repaint();
  }

  // paint method
  public void paint(Graphics g)
  {
    // If the limits haven't been set, loop over all values to
    // find the extremes
    DecimalFormat form = new DecimalFormat("###,##0.0");
    if(!limitsset)
    {
      minFit= 0.0;
      maxFit= 0.0;
      for(int i= 0; i < age; i++)
      {
	if(mode == THREE)
	{
	  maxFit= Math.max(bests[i], maxFit);
	  maxFit= Math.max(worsts[i], maxFit);
	}
	maxFit= Math.max(averages[i], maxFit);
      }
      maxFit += (double) ((int) (maxFit * .1 + .5));
    }

    // Find out, how much space we need to the left
    String num= "" + maxFit;
    int left= metric.stringWidth(num);
    num= "" + minFit;
    left= Math.max(left, metric.stringWidth(num));
    num= "0";
    left= Math.max(left, metric.stringWidth(num));

    // Construct a Rectangle that holds the space we can draw in
    Rectangle big= getBounds();
    Rectangle inb= new Rectangle(left + 2, 0, big.width - left -2, big.height - metric.getHeight() - 2);

    // Clear the Canvas
    g.clearRect(big.x, big.y, big.width -1, big.height -1);
    setForeground(Color.black);
    g.drawLine(inb.x, inb.y, inb.x, inb.y + inb.height);
    g.drawLine(inb.x, inb.height, inb.x + inb.width, inb.height);

    // Plot the units of the fitness
    num= "" + minFit;
    g.drawString(num, 1, inb.y + inb.height  );
    num= "" + form.format(maxFit);
    g.drawString(num, 1, inb.y + metric.getAscent());
    if((int) (maxFit / 2) != 0)
    {
      num= "" + (int) (maxFit / 2);
      g.drawString(num, 1, inb.y + (inb.height / 2)+ (metric.getAscent() / 2));
    }

    // Plot the units of the generations
    num= "" + 0;
    g.drawString(num, inb.x, inb.y + inb.height + fheight);
    num= "" + numGens;
    left= metric.stringWidth(num);
    g.drawString(num, inb.x + inb.width - left, inb.y + inb.height + fheight);
    left= metric.stringWidth(title);
    g.drawString(title, inb.x + inb.width/2 - left/2, inb.height + fheight);

    int x, y, ox= -1, oy= -1;

    if(mode == THREE)
    {
      ox= -1;
      g.setColor(Color.green);
      for(int i= 0; i < age; i++)
      {
	y= inb.y + inb.height - (int) ((bests[i] * (double) inb.height) / (maxFit - minFit));
	x= inb.x + (inb.width * i) / numGens;
	g.drawOval(x -1, y -1, 3, 3);
	if(ox != -1)
	  g.drawLine(ox, oy, x, y);
	ox= x; oy= y;
      }

      ox= -1;
      g.setColor(Color.red);
      for(int i= 0; i < age; i++)
      {
	y= inb.y + inb.height - (int) ((worsts[i] * (double) inb.height) / (maxFit - minFit));
	x= inb.x + (inb.width * i) / numGens;
	g.drawOval(x -1, y -1, 3, 3);
	if(ox != -1)
	g.drawLine(ox, oy, x, y);
	ox= x; oy= y;
      }
    }

    ox= -1;
    g.setColor(Color.black);
    for(int i= 0; i < age; i++)
    {
      y= inb.y + inb.height - (int) ((averages[i] * (double) inb.height) / (maxFit - minFit)) ;
      x= inb.x + (inb.width * i) / numGens;
      g.drawOval(x -1 , y -1 , 3, 3);
      if(ox != -1)
	g.drawLine(ox, oy, x, y);
      ox= x; oy= y;
    }
  }

}