package stockMarketApplet.GUI;

import java.awt.*;
import java.util.*;
import java.applet.*;

import stockMarketApplet.util.Util;
import stockMarketApplet.market.agents.technicalTrader.basics.*;

// Displays the code of an Individual as tree. Uses scrollbars and provides
// three methods of displaying the nodes: as black boxes, three letter acronyms
// or small black boxes.
public class TreePanel extends Panel {
  private GridBagLayout layouter;
  private TreeCanvas disp;

  private Scrollbar hbar, vbar;

  private Button butt;

  private Program code;
  private int xoff, yoff;

  //  Create a TreePanel.
  public TreePanel()
  {
    layouter= new GridBagLayout();
    this.setLayout(layouter);
    Util util = new Util();

    yoff= 0;
    xoff= 50;

    disp= new TreeCanvas();
    disp.setXOff(xoff);
    disp.setYOff(yoff);
    vbar= new Scrollbar(Scrollbar.VERTICAL, yoff, 5, 0, 99);
    hbar= new Scrollbar(Scrollbar.HORIZONTAL, xoff, 5, 0, 99);
    butt= new Button();

    util.constrain(this, disp, 0, 0, 1, 1,
	      GridBagConstraints.BOTH, GridBagConstraints.CENTER,
    	      1.0, 1.0, 0, 0, 0, 0);
    util.constrain(this, vbar, 1, 0, 1, 1,
	      GridBagConstraints.VERTICAL, GridBagConstraints.EAST,
    	      0.0, 1.0, 0, 0, 0, 0);
    util.constrain(this, hbar, 0, 1, 1, 1,
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTH,
    	      1.0, 0.0, 0, 0, 0, 0);
    util.constrain(this, butt, 1, 1, 1, 1,
	      GridBagConstraints.NONE, GridBagConstraints.SOUTHEAST,
    	      0.0, 0.0, 0, 0, 0, 0);
  }

  // Set the root symbol of the code that shall be drawn
  public void setCode(Program root)
  {
    code= root;
    disp.setCode(code);
    center();
  }

  // Center the display on the root symbol
  private void center()
  {
    xoff = 50;
    yoff = 0;

    disp.setXOff(xoff);
    disp.setYOff(yoff);

    hbar.setValue(xoff);
    vbar.setValue(yoff);
  }

  // Reset Display
  public void clear()
  {
    disp.clear();
    center();
  }

  // Handle user input
  public boolean handleEvent(Event e)
  {
    if(e.target == hbar)
    {
      switch(e.id)
      {
      case Event.SCROLL_LINE_UP:
      case Event.SCROLL_LINE_DOWN:
      case Event.SCROLL_PAGE_UP:
      case Event.SCROLL_PAGE_DOWN:
      case Event.SCROLL_ABSOLUTE:
	xoff = ((Integer) e.arg).intValue();
        disp.setXOff(xoff);
	break;
      }
      return true;
    } else if(e.target == vbar)
    {
      switch(e.id)
      {
      case Event.SCROLL_LINE_UP:
      case Event.SCROLL_LINE_DOWN:
      case Event.SCROLL_PAGE_UP:
      case Event.SCROLL_PAGE_DOWN:
      case Event.SCROLL_ABSOLUTE:
	yoff = ((Integer) e.arg).intValue();
        disp.setYOff(yoff);
	break;
      }
      return true;
    } else if(e.target == butt)
    {
      if(e.id == Event.ACTION_EVENT)
      {
	disp.toggleMode();
	center();
	return true;
      }
    }
    return false;
  }

}

class TreeCanvas extends Canvas {
  public final static int TEXT = 0;
  public final static int GRAPH = 1;
  public final static int SMALLGRAPH = 2;

  private Font smallfont;
  private int fsize;
  private int xoff, yoff;
  private int xper, yper;

  private Program code;
  private Rectangle virt, vis;
  private int mode;

  private FontMetrics fm;

  public TreeCanvas()
  {
    smallfont= new Font("Courier",Font.BOLD,14);
    setFont(smallfont);
    fsize= smallfont.getSize();

    setBackground(Color.white);
    setForeground(Color.black);

    mode = GRAPH;

    virt = new Rectangle();
    vis = new Rectangle();
    xoff= 0;
    yoff= 0;
  }

  public void setCode(Program root)
  {
    code= root;
    repaint();
  }

  public void setXOff(int o)
  {
    xper= o;
    repaint();
  }

  public void setYOff(int o)
  {
    yper= o;
    repaint();
  }

  public void setMode(int i)
  {
    mode= i;
    repaint();
  }

  public void toggleMode()
  {
    switch(mode)
    {
      case GRAPH:
	mode = TEXT;
	break;
      case TEXT:
	mode = SMALLGRAPH;
	break;
      case SMALLGRAPH:
	mode = GRAPH;
	break;
    }
    repaint();
  }

  public void paint(Graphics g)
  {
    if(code == null)
      return;

    int lgt = code.depth();
    int lvs = code.leaves();

    fm = getFontMetrics(getFont());

    vis= getBounds();
    g.clearRect(0, 0, vis.width, vis.height);

    vis.x= 0;
    vis.y= 0;

    int h;

    switch(mode)
    {
      case SMALLGRAPH:
	h = 8;
	break;
      default:
	h = 30;
	break;
    }

    virt.x= vis.x;
    virt.y= vis.y;

    // virt.width = vis.width * (1 + code.depth() / 4);
    if((h * lvs) >= vis.width)
      virt.width = h * lvs;
    else
      virt.width = vis.width;

    //virt.height = vis.height * (1 + code.depth() / 4);
    if((h * lgt) >= vis.height)
      virt.height = h * lgt;
    else
      virt.height = vis.height;

    xoff= ((virt.width - vis.width) * xper) / 100;
    yoff= ((virt.height - vis. height) * yper) / 100;

    drawTree(g, virt, code, h);
  }

  public void drawTree(Graphics g, Rectangle area, Program root, int h)
  {
    switch(mode)
    {
      case GRAPH:
      case SMALLGRAPH:
	g.fillRect(area.x + (area.width / 2) - (h / 2) - xoff,
		   area.y + h/2 - yoff,
		   h - h/5, h/2);
	break;

      case TEXT:
	String str;
	if(root.getExpression().get(0) != null)
	{
	  str= root.getExpression().get(0).toString();
	} else
	{
	  char[] mistjava= new char[2];
	  mistjava[0] = root.getExpression().get(0).toString().charAt(0);
	  try {
	    mistjava[1] = root.getExpression().get(0).toString().charAt(1);
	  }
	  catch(StringIndexOutOfBoundsException se)
	  {
	    ;
	  }
	  str = new String(mistjava);
	}
	int sx = ((h - fm.stringWidth(str)) / 2);
	int sy = fm.getAscent();
	g.drawString(str, area.x + (area.width / 2) - (h / 2) - xoff + sx,
		   area.y + h/2 - yoff + sy);
	break;
    }

    if(root.getDecisionSet().contains(root.getExpression().get(0)))
      return;

    else if(root.getFunctionSet().contains(root.getExpression().get(0)))
    {
      Program node = root;

      Rectangle newarea= new Rectangle();

      newarea.x = area.x;
      newarea.y = area.y + h;
      newarea.height = area.height - h;
      newarea.width = 0;
      int oldwidth = 0;

      for(short i= 0; i < node.getNumberArgs(); i++)
      {
	newarea.x = area.x + oldwidth;  // + (area.width * node.getArgument(i).leaves() / node.leaves() );
	newarea.width = area.width * node.getArgument(i).leaves() / node.leaves();

	g.drawLine(area.x + (area.width / 2) - xoff,
		   area.y + h - yoff,
		   newarea.x + (newarea.width / 2) - xoff,
		   newarea.y + h/2 - yoff);
	drawTree(g, newarea, node.getArgument(i), h);

	oldwidth += newarea.width;
      }
    }
  }

  public void clear()
  {
    code = null;
    mode = GRAPH;
    repaint();
  }

}
