package stockMarketApplet.market.agents.technicalTrader.basics;

import java.util.Random;
import java.util.Calendar;
import stockMarketApplet.util.Util;

public class Generation extends Object {
  public final static byte BEST  = 0;
  public final static byte NEXT  = 1;
  public final static byte PREV  = 2;
  public final static byte WORST = 3;
  private Individual[] population;
  private double averageFit;
  private double adjustedFitSum;
  private int averageSize;
  private int selected;
  private Random generator;
  private boolean valid;
  private boolean isSorted;
  public Generation(Individual[] pop)  {
    population = pop;
    generator = new Random();
    Calendar date = Calendar.getInstance();
    generator.setSeed(date.get(Calendar.MILLISECOND));
    valid = false;
    isSorted = false;
    selected = population.length -1;
  }
  public Individual getIndividual(int n)  {
    try
    {
      return population[n];
    }
    catch (ArrayIndexOutOfBoundsException e)
    {
      return population[ population.length -1 ];
    }
  }
  public Individual getSelected()  {
    try
    {
      return population[selected];
    }
    catch (ArrayIndexOutOfBoundsException e)
    {
      return population[ population.length -1 ];
    }
  }
  public void setSelected(int mode)  {
    switch(mode)
    {
      case BEST:
	selected = population.length -1;
	break;
      case WORST:
	selected = 0;
	break;
      case NEXT:
	if(selected < (population.length -1))
	  selected++;
	break;
      case PREV:
	if(selected > 0)
	  selected--;
	break;
    }
  }
  public Individual[] getPopulation()  {
    return population;
  }
  public void setPopulation( Individual[] pop){
    this.population = pop;
  }
  public int getPopulationSize()  {
    return population.length;
  }
  public boolean getValid()  {
    return valid;
  }
  private void setValid(boolean b)  {
    valid = b;
  }
  public void computeStats()  {
    double sum = 0.0;
    adjustedFitSum  = 0.0;
//    averageSize= 0;
    for(int i= 0; i < population.length; i++)
    {
      sum += population[i].getFitness();
      adjustedFitSum += 1 / (1 + population[i].getFitness());
//      averageSize += population[i].size();
    }
    sum /= (double) population.length;
//    averageSize /= population.length;
    averageFit = sum;
    setValid(true);
  }
  public void computeAvgSize(){
  averageSize = 0;
  for(int i= 0; i < population.length; i++)
  {
    averageSize += population[i].size();
  }
    averageSize /= population.length;
}

  private void sortPopulation()  {
    Util.QuickSort(population, 0, population.length - 1);
    isSorted = true;
  }
  public double getAverageFitness()  {
    return averageFit;
  }
  public int getAverageSize()  {
    return averageSize;
  }
  public double getAdjustedFitSum()  {
    return adjustedFitSum;
  }
  public Individual getBestIndividual()  {
    if (!isSorted){
      sortPopulation();
    }
    return population[0];
  }
  public void setBestIndividual(Individual ind){
      this.population[0] = (Individual) ind.clone();
  }
  public Individual getBestRandInd(int n)  {
    if (!isSorted){
      sortPopulation();
    }
    try {
      return population[(int) (n * generator.nextDouble())];
    }
    catch (ArrayIndexOutOfBoundsException e) {
      return population[(int) (population.length * generator.nextDouble())];
    }
  }
  public Individual getWorstIndividual()  {
    if (!isSorted){
      sortPopulation();
    }
    return population[ population.length -1 ];
  }
  public Individual getWorstRandomIndividual(int n)  {
    try {
      return population[ population.length - 1 - (int) (n * generator.nextDouble()) ];
    }
    catch (ArrayIndexOutOfBoundsException e) {
      return population[ population.length - 1 - (int) (population.length * generator.nextDouble()) ];
    }
  }
  public Individual getRandInd()  {
    try    {
      return population[(int)(population.length * generator.nextDouble())];
    }
    catch (ArrayIndexOutOfBoundsException e) {
      return population[population.length -1 ];
    }
  }
  public Individual getFitPropInd()  {
    double d = Math.abs(generator.nextDouble());
    int i = population.length;
    double s = 0.0;
    try
    {
      while(s < d)
      {
        // Error in the array when i = 0
	s += (1.0 / (1.0 + population[i-1].getFitness())) / adjustedFitSum;
	i--;
      }
      return population[i];
    }
    catch (ArrayIndexOutOfBoundsException e)
    {
      System.out.println("Index of the individual: " + i);
      System.out.println("OutOfBounds in getFitPropIndividual");
      return population[ 0 ];
    }
  }
  // Returns true if an individual with a fitness of zero has been found during judge()
  public boolean checkTerminate()  {
    return false;
  }
  public void dropPopulation()  {
    population = null;
  }
  public void clear()  {
    for(int p= 0; p < population.length; p++)
	population[p].resetFitness();
  }

}